import unittest
from pathlib import Path

from dbtmetabase.format import Filter, NullValue, dump_yaml, safe_description, safe_name


class TestFormat(unittest.TestCase):
    def test_filter(self):
        self.assertTrue(
            Filter(
                include=("alpHa", "bRavo"),
            ).match("Alpha")
        )
        self.assertTrue(Filter().match("Alpha"))
        self.assertTrue(Filter().match(""))
        self.assertFalse(
            Filter(
                include=("alpHa", "bRavo"),
                exclude=("alpha",),
            ).match("Alpha")
        )
        self.assertFalse(
            Filter(
                exclude=("alpha",),
            ).match("Alpha")
        )
        self.assertTrue(Filter(include="alpha").match("Alpha"))
        self.assertFalse(Filter(exclude="alpha").match("Alpha"))

    def test_null_value(self):
        self.assertIsNotNone(NullValue)
        self.assertFalse(NullValue)
        self.assertIs(NullValue, NullValue)

    def test_safe_name(self):
        self.assertEqual(
            "somebody_s_2_collections_",
            safe_name("Somebody's 2 collections!"),
        )
        self.assertEqual(
            "somebody_s_2_collections_",
            safe_name("somebody_s_2_collections_"),
        )
        self.assertEqual("", safe_name(""))

    def test_safe_description(self):
        self.assertEqual(
            "Depends on\n\nQuestion ( #2 )!",
            safe_description("Depends on\n\nQuestion {{ #2 }}!"),
        )
        self.assertEqual(
            "Depends on\n\nQuestion ( #2 )!",
            safe_description("Depends on\n\nQuestion ( #2 )!"),
        )
        self.assertEqual(
            "Depends on\n\nQuestion { #2 }!",
            safe_description("Depends on\n\nQuestion { #2 }!"),
        )
        self.assertEqual(
            "(start_date) - cast((rolling_days))",
            safe_description("{{start_date}} - cast({{rolling_days}})"),
        )

    def test_dump_yaml(self):
        path = Path("tests") / "tmp" / "test_dump_yaml.yml"
        with open(path, "w", encoding="utf-8") as f:
            dump_yaml(
                data={
                    "root": {
                        "attr1": "val1\nend",
                        "attr2": ["val2", "val3"],
                    },
                },
                stream=f,
            )
        with open(path, "r", encoding="utf-8") as f:
            self.assertEqual(
                """root:
  attr1: 'val1

    end'
  attr2:
    - val2
    - val3
""",
                f.read(),
            )



# pylint: disable=protected-access,no-member

import unittest

from ._mocks import MockDbtMetabase


class TestModels(unittest.TestCase):
    def setUp(self):
        self.c = MockDbtMetabase()
        self.c._ModelsMixin__SYNC_PERIOD = 1  # type: ignore

    def test_export(self):
        self.c.export_models(
            metabase_database="unit_testing",
            skip_sources=True,
            sync_timeout=0,
        )

    def test_build_lookups(self):
        expected_tables = [
            "PUBLIC.CUSTOMERS",
            "PUBLIC.ORDERS",
            "PUBLIC.RAW_CUSTOMERS",
            "PUBLIC.RAW_ORDERS",
            "PUBLIC.RAW_PAYMENTS",
            "PUBLIC.STG_CUSTOMERS",
            "PUBLIC.STG_ORDERS",
            "PUBLIC.STG_PAYMENTS",
        ]
        actual_tables = self.c._ModelsMixin__get_tables(database_id="2")  # type: ignore
        self.assertEqual(expected_tables, list(actual_tables.keys()))

        expected_columns = [
            [
                "CUSTOMER_ID",
                "FIRST_NAME",
                "LAST_NAME",
                "FIRST_ORDER",
                "MOST_RECENT_ORDER",
                "NUMBER_OF_ORDERS",
                "CUSTOMER_LIFETIME_VALUE",
            ],
            [
                "ORDER_ID",
                "CUSTOMER_ID",
                "ORDER_DATE",
                "STATUS",
                "CREDIT_CARD_AMOUNT",
                "COUPON_AMOUNT",
                "BANK_TRANSFER_AMOUNT",
                "GIFT_CARD_AMOUNT",
                "AMOUNT",
            ],
            ["ID", "FIRST_NAME", "LAST_NAME"],
            ["ID", "USER_ID", "ORDER_DATE", "STATUS"],
            ["ID", "ORDER_ID", "PAYMENT_METHOD", "AMOUNT"],
            ["CUSTOMER_ID", "FIRST_NAME", "LAST_NAME"],
            ["ORDER_ID", "CUSTOMER_ID", "ORDER_DATE", "STATUS"],
            ["PAYMENT_ID", "ORDER_ID", "PAYMENT_METHOD", "AMOUNT"],
        ]
        for table, columns in zip(expected_tables, expected_columns):
            self.assertEqual(columns, list(actual_tables[table]["fields"].keys()))
